@@ -10,21 +10,37 @@
 #include "interface.h"
 #include "support.h"
 
-#include "mfsk.h"
+#include "main.h"
+#include "trx.h"
 #include "wf.h"
 #include "macro.h"
 #include "prefs.h"
 #include "qsodata.h"
+#include "log.h"
 
 static GtkWidget *txfileselectwin = NULL;
 static GtkWidget *rxfileselectwin = NULL;
 static gchar *popupselection = NULL;
 
+/* ---------------------------------------------------------------------- */
+
+gboolean
+on_appwindow_delete_event              (GtkWidget       *widget,
+                                        GdkEvent        *event,
+                                        gpointer         user_data)
+{
+	trx_set_state(TRX_STATE_ABORT);
+	gtk_main_quit();
+
+	return TRUE;
+}
+
+
 void
 on_exit1_activate                      (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-	stop_trx(TRUE);
+	trx_set_state(TRX_STATE_ABORT);
 	gtk_main_quit();
 }
 
@@ -51,6 +67,7 @@ on_about1_activate                     (GtkMenuItem     *menuitem,
 	gtk_widget_show(aboutwindow);
 }
 
+/* ---------------------------------------------------------------------- */
 
 gboolean
 on_waterfall_motion_notify_event       (GtkWidget       *widget,
@@ -58,12 +75,9 @@ on_waterfall_motion_notify_event       (GtkWidget       *widget,
                                         gpointer         user_data)
 {
 	Waterfall *wfall;
-	gint width;
 
 	wfall = WATERFALL(lookup_widget(appwindow, "waterfall"));
-
-	width = (NumTones - 1) * ToneSpacing * WATERFALL_WIDTH / WF_SPAN;
-	waterfall_setmarker1(wfall, event->x, event->x + width);
+	waterfall_setmarker1(wfall, event->x);
 
 	return FALSE;
 }
@@ -77,7 +91,7 @@ on_waterfall_leave_notify_event        (GtkWidget       *widget,
 	Waterfall *wfall;
 
 	wfall = WATERFALL(lookup_widget(appwindow, "waterfall"));
-	waterfall_setmarker1(wfall, -1, -1);
+	waterfall_setmarker1(wfall, -1);
 
 	return FALSE;
 }
@@ -92,39 +106,18 @@ on_waterfall_button_press_event        (GtkWidget       *widget,
 
 	/* button 3 over waterfall toggles waterfall pause */
 	if (event->button == 3) {
-		WFpause = !WFpause;
+		wf_togglepause();
 		return FALSE;
 	}
 
-	freq = WF_XTOF(event->x);
+	/* any other button sets trx frequency */
+	freq = wf_xtof(event->x);
 	trx_set_freq(freq);
 
 	return FALSE;
 }
 
-void
-on_freqspinbutton_changed              (GtkEditable     *editable,
-                                        gpointer         user_data)
-{
-	GtkSpinButton *spin;
-	gfloat freq;
-
-	spin = GTK_SPIN_BUTTON(editable);
-	freq = gtk_spin_button_get_value_as_float(spin);
-	trx_set_freq(freq);
-}
-
-
-gboolean
-on_appwindow_delete_event              (GtkWidget       *widget,
-                                        GdkEvent        *event,
-                                        gpointer         user_data)
-{
-	stop_trx(TRUE);
-	gtk_main_quit();
-
-	return FALSE;
-}
+/* ---------------------------------------------------------------------- */
 
 gboolean
 on_txtext_key_press_event              (GtkWidget       *widget,
@@ -133,12 +126,16 @@ on_txtext_key_press_event              (GtkWidget       *widget,
 {
 	GtkText *text = GTK_TEXT(widget);
 
+	/* ignore if CTRL or ALT is pressed (reserved for accelerators) */
+	if ((event->state & GDK_CONTROL_MASK) || (event->state & GDK_MOD1_MASK))
+		return FALSE;
+
 	switch (event->keyval) {
 	case GDK_Return:
 		send_char('\n');
 		break;
 	case GDK_BackSpace:
-		typeahead_del_char();
+		trx_put_tx_char(8);
 		gtk_text_backward_delete(text, 1);
 		gtk_text_backward_delete(text, 1);
 		gtk_text_insert(text, NULL, NULL, NULL, "_", -1);
@@ -161,12 +158,26 @@ on_rxtext_key_release_event            (GtkWidget       *widget,
 	return FALSE;
 }
 
+/* ---------------------------------------------------------------------- */
+
+void
+on_freqspinbutton_changed              (GtkEditable     *editable,
+                                        gpointer         user_data)
+{
+	GtkSpinButton *spin;
+	gfloat freq;
+
+	spin = GTK_SPIN_BUTTON(editable);
+	freq = gtk_spin_button_get_value_as_float(spin);
+	trx_set_freq(freq);
+}
+
 
 void
 on_afcbutton_toggled                   (GtkToggleButton *togglebutton,
                                         gpointer         user_data)
 {
-	AFCon = gtk_toggle_button_get_active(togglebutton);
+	trx_set_afc(gtk_toggle_button_get_active(togglebutton));
 }
 
 
@@ -174,10 +185,19 @@ void
 on_squelchbutton_toggled               (GtkToggleButton *togglebutton,
                                         gpointer         user_data)
 {
-	SQUELCHon = gtk_toggle_button_get_active(togglebutton);
+	trx_set_squelch(gtk_toggle_button_get_active(togglebutton));
 }
 
 
+void
+on_reversebutton_toggled               (GtkToggleButton *togglebutton,
+                                        gpointer         user_data)
+{
+	trx_set_reverse(gtk_toggle_button_get_active(togglebutton));
+}
+
+/* ---------------------------------------------------------------------- */
+
 void
 on_macrobutton_clicked                 (GtkButton       *button,
                                         gpointer         user_data)
@@ -235,6 +255,7 @@ on_macroconfigwindow_delete_event      (GtkWidget       *widget,
 	return FALSE;
 }
 
+/* ---------------------------------------------------------------------- */
 
 void
 on_propertybox_apply                   (GnomePropertyBox *gnomepropertybox,
@@ -246,19 +267,16 @@ on_propertybox_apply                   (GnomePropertyBox *gnomepropertybox,
 		apply_config(GTK_WIDGET(gnomepropertybox));
 }
 
+/* ---------------------------------------------------------------------- */
 
 void
 on_pausebutton_toggled                 (GtkToggleButton *togglebutton,
                                         gpointer         user_data)
 {
-	gboolean state;
+	gboolean b = gtk_toggle_button_get_active(togglebutton);
 
-	state = gtk_toggle_button_get_active(togglebutton);
-
-	if (state)
-		WFpause = TRUE;
-	else
-		WFpause = FALSE;
+	if (b && trx_get_state() != TRX_STATE_ABORT)
+		trx_set_state(TRX_STATE_PAUSE);
 }
 
 
@@ -266,14 +284,9 @@ void
 on_rxbutton_toggled                    (GtkToggleButton *togglebutton,
                                         gpointer         user_data)
 {
-	gboolean state;
+	gboolean b = gtk_toggle_button_get_active(togglebutton);
 
-	state = gtk_toggle_button_get_active(togglebutton);
-
-	if (state)
-		start_rx();
-	else
-		stop_trx(FALSE);
+	if (b)	trx_set_state(TRX_STATE_RX);
 }
 
 
@@ -281,14 +294,16 @@ void
 on_txbutton_toggled                    (GtkToggleButton *togglebutton,
                                         gpointer         user_data)
 {
-	gboolean state;
+	GtkWidget *widget;
+	gboolean b = gtk_toggle_button_get_active(togglebutton);
 
-	state = gtk_toggle_button_get_active(togglebutton);
+	if (b) {
+		trx_set_state(TRX_STATE_TX);
 
-	if (state)
-		start_tx();
-	else
-		stop_trx(FALSE);
+		/* set focus to the tx text widget */
+		widget = lookup_widget(appwindow, "txtext");
+		gtk_widget_grab_focus(widget);
+	}
 }
 
 
@@ -296,13 +311,44 @@ void
 on_abortbutton_clicked                 (GtkButton       *button,
                                         gpointer         user_data)
 {
-	if (TXon) {
-		stop_trx(TRUE);
-		push_button("pausebutton");
+	gint state = trx_get_state();
+
+	if (state != TRX_STATE_ABORT && state != TRX_STATE_PAUSE) {
+		trx_set_state_wait(TRX_STATE_ABORT);
+		/* clear tx typeahead buffer */
+		while (trx_get_tx_char() != -1);
+		/* switch to rx */
+		trx_set_state_wait(TRX_STATE_RX);
+		push_button("rxbutton");
 	}
 }
 
 
+static gint
+tune_timeout_cb                        (gpointer unused)
+{
+	gdk_threads_enter();
+	if (trx_get_state() == TRX_STATE_TUNE)
+		push_button("rxbutton");
+	gdk_threads_leave();
+
+	return FALSE;
+}
+
+
+void
+on_tunebutton_toggled                  (GtkToggleButton *togglebutton,
+                                        gpointer         user_data)
+{
+	gboolean b = gtk_toggle_button_get_active(togglebutton);
+
+	if (b) {
+		trx_set_state(TRX_STATE_TUNE);
+		gtk_timeout_add(30000, tune_timeout_cb, NULL);
+	}
+}
+
+/* ---------------------------------------------------------------------- */
 
 gboolean
 on_rxtext_button_press_event           (GtkWidget       *widget,
@@ -323,6 +369,7 @@ on_rxtext_button_press_event           (GtkWidget       *widget,
 	bevent.button.type = GDK_BUTTON_RELEASE;
 	gtk_widget_event(widget, &bevent);
 
+	/* find the word */
 	pos = gtk_editable_get_position(edit);
 
 	if (isspace(GTK_TEXT_INDEX(text, pos)))
@@ -344,9 +391,11 @@ on_rxtext_button_press_event           (GtkWidget       *widget,
 	if (stop == len)
 		stop--;
 
+	/* save the selected text */
 	g_free(popupselection);
 	popupselection = gtk_editable_get_chars(edit, start, stop);
 
+	/* select it */
 	gtk_editable_select_region(edit, start, stop);
 
 	/* pop the menu up */
@@ -457,21 +506,7 @@ on_qsoentry_changed                    (GtkEditable     *editable,
 	set_qsotime();
 }
 
-
-void
-on_tunebutton_pressed                  (GtkButton       *button,
-                                        gpointer         user_data)
-{
-	start_tune();
-}
-
-
-void
-on_tunebutton_released                 (GtkButton       *button,
-                                        gpointer         user_data)
-{
-	stop_tune();
-}
+/* ---------------------------------------------------------------------- */
 
 gboolean
 on_txtext_button_press_event           (GtkWidget       *widget,
@@ -489,8 +524,8 @@ on_txtext_button_press_event           (GtkWidget       *widget,
 		targets_atom = gdk_atom_intern("STRING", FALSE);
 
 	/* And request the "STRING" target for the primary selection */
-        gtk_selection_convert(widget, GDK_SELECTION_PRIMARY, targets_atom,
-			      GDK_CURRENT_TIME);
+        gtk_selection_convert(widget, GDK_SELECTION_PRIMARY,
+			      targets_atom, GDK_CURRENT_TIME);
 
 	return TRUE;
 }
@@ -519,6 +554,7 @@ on_txtext_selection_received           (GtkWidget       *widget,
 	send_string(data->data);
 }
 
+/* ---------------------------------------------------------------------- */
 
 void
 on_send_file1_activate                 (GtkMenuItem     *menuitem,
@@ -567,7 +603,8 @@ on_clear_tx_window1_activate           (GtkMenuItem     *menuitem,
 	editable = GTK_EDITABLE(lookup_widget(appwindow, "txtext"));
 	gtk_editable_delete_text(editable, 0, -1);
 
-	typeahead_clear();
+	/* clear tx typeahead buffer */
+	while (trx_get_tx_char() != -1);
 }
 
 
@@ -581,6 +618,7 @@ on_clear_rx_window1_activate           (GtkMenuItem     *menuitem,
 	gtk_editable_delete_text(editable, 0, -1);
 }
 
+/* ---------------------------------------------------------------------- */
 
 void
 on_macroconfig1_activate               (GtkMenuItem     *menuitem,
@@ -677,6 +715,7 @@ on_macroconfig12_activate              (GtkMenuItem     *menuitem,
 	macroconfig(12);
 }
 
+/* ---------------------------------------------------------------------- */
 
 void
 on_txfileselect_ok_button_clicked      (GtkButton       *button,
@@ -780,4 +819,143 @@ on_rxfileselection_delete_event        (GtkWidget       *widget,
 	return FALSE;
 }
 
+/* ---------------------------------------------------------------------- */
+
+void
+on_mfsk1_activate                      (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	GtkCheckMenuItem *item = GTK_CHECK_MENU_ITEM(menuitem);
+        Waterfall *wfall;
+	gint state;
+
+        wfall = WATERFALL(lookup_widget(appwindow, "waterfall"));
+
+	if (item->active && trx_get_mode() != MODE_MFSK16) {
+		state = trx_get_state();
+		trx_set_state_wait(TRX_STATE_ABORT);
+		trx_set_mode(MODE_MFSK16);
+		trx_set_state_wait(state);
+		gtk_widget_set_sensitive(lookup_widget(appwindow, "squelchbutton"), TRUE);
+		waterfall_setcenter(wfall, FALSE);
+	}
+}
+
+
+void
+on_rtty1_activate                      (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	GtkCheckMenuItem *item = GTK_CHECK_MENU_ITEM(menuitem);
+        Waterfall *wfall;
+	gint state;
+
+        wfall = WATERFALL(lookup_widget(appwindow, "waterfall"));
+
+	if (item->active && trx_get_mode() != MODE_RTTY) {
+		state = trx_get_state();
+		trx_set_state_wait(TRX_STATE_ABORT);
+		trx_set_mode(MODE_RTTY);
+		trx_set_state_wait(state);
+		trx_set_metric(0.0);
+		gtk_widget_set_sensitive(lookup_widget(appwindow, "squelchbutton"), FALSE);
+		waterfall_setcenter(wfall, FALSE);
+	}
+}
+
+
+void
+on_throb1_activate                     (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	GtkCheckMenuItem *item = GTK_CHECK_MENU_ITEM(menuitem);
+        Waterfall *wfall;
+	gint state;
+
+        wfall = WATERFALL(lookup_widget(appwindow, "waterfall"));
+
+	if (item->active && trx_get_mode() != MODE_THROB1) {
+		state = trx_get_state();
+		trx_set_state_wait(TRX_STATE_ABORT);
+		trx_set_mode(MODE_THROB1);
+		trx_set_state_wait(state);
+		trx_set_metric(0.0);
+		gtk_widget_set_sensitive(lookup_widget(appwindow, "squelchbutton"), FALSE);
+		waterfall_setcenter(wfall, TRUE);
+	}
+}
+
+
+void
+on_throb2_activate                     (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	GtkCheckMenuItem *item = GTK_CHECK_MENU_ITEM(menuitem);
+        Waterfall *wfall;
+	gint state;
+
+        wfall = WATERFALL(lookup_widget(appwindow, "waterfall"));
+
+	if (item->active && trx_get_mode() != MODE_THROB2) {
+		state = trx_get_state();
+		trx_set_state_wait(TRX_STATE_ABORT);
+		trx_set_mode(MODE_THROB2);
+		trx_set_state_wait(state);
+		trx_set_metric(0.0);
+		gtk_widget_set_sensitive(lookup_widget(appwindow, "squelchbutton"), FALSE);
+		waterfall_setcenter(wfall, TRUE);
+	}
+}
+
+
+void
+on_throb4_activate                     (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	GtkCheckMenuItem *item = GTK_CHECK_MENU_ITEM(menuitem);
+        Waterfall *wfall;
+	gint state;
+
+        wfall = WATERFALL(lookup_widget(appwindow, "waterfall"));
+
+	if (item->active && trx_get_mode() != MODE_THROB4) {
+		state = trx_get_state();
+		trx_set_state_wait(TRX_STATE_ABORT);
+		trx_set_mode(MODE_THROB4);
+		trx_set_state_wait(state);
+		trx_set_metric(0.0);
+		gtk_widget_set_sensitive(lookup_widget(appwindow, "squelchbutton"), FALSE);
+		waterfall_setcenter(wfall, TRUE);
+	}
+}
+
+/* ---------------------------------------------------------------------- */
+
+void
+on_mag1button_toggled                  (GtkToggleButton *togglebutton,
+                                        gpointer         user_data)
+{
+	if (gtk_toggle_button_get_active(togglebutton))
+		init_wf(2048, trx_get_freq() + trx_get_bandwidth() / 2.0);
+}
+
+
+void
+on_mag2button_toggled                  (GtkToggleButton *togglebutton,
+                                        gpointer         user_data)
+{
+	if (gtk_toggle_button_get_active(togglebutton))
+		init_wf(4096, trx_get_freq() + trx_get_bandwidth() / 2.0);
+}
+
+
+void
+on_mag4button_toggled                  (GtkToggleButton *togglebutton,
+                                        gpointer         user_data)
+{
+	if (gtk_toggle_button_get_active(togglebutton))
+		init_wf(8192, trx_get_freq() + trx_get_bandwidth() / 2.0);
+}
+
+/* ---------------------------------------------------------------------- */
 
