@@ -1,6 +1,8 @@
 #ifndef _PREFS_H
 #define _PREFS_H 1
 
+#include <gnome.h>
+
 struct preferences {
 	gchar mycall[64];
 	gchar myname[64];
@@ -19,8 +21,6 @@ struct preferences {
 
 	gfloat txoffset;
 
-	struct macro macros[NUMMACROS];
-
 	gboolean afc;
 	gboolean squelch;
 
@@ -33,6 +33,22 @@ struct preferences {
 
 	gchar pttdev[64];
 	gboolean pttinv;
+
+	gfloat sensitivity;
+	gint window;
+
+	gfloat rttyshift;
+	gfloat rttybaud;
+	gint rttybits;
+	gint rttyparity;
+	gint rttystop;
+	gboolean rttyreverse;
+	gboolean rttymsb;
+
+	gchar qsobands[256];
+
+	gchar timefmt[64];
+	gchar datefmt[64];
 };
 
 extern struct preferences prefs;
@@ -45,4 +61,3 @@ extern void fill_config(GtkWidget *);
 extern GdkFont *rxfont, *txfont;
 
 #endif
-
