@@ -1,8 +1,8 @@
 #ifndef _SND_H
 #define _SND_H
 
-extern int sound_open_for_write(void);
-extern int sound_open_for_read(void);
+extern int sound_open_for_write(int srate);
+extern int sound_open_for_read(int srate);
 
 extern void sound_close(void);
 
